﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Ivi.Visa.Interop;
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//''  © Agilent Technologies, Inc. 2013
//''
//'' You have a royalty-free right to use, modify, reproduce and distribute
//'' the Sample Application Files (and/or any modified version) in any way
//'' you find useful, provided that you agree that Agilent Technologies has no
//'' warranty,  obligations or liability for any Sample Application Files.
//''
//'' Agilent Technologies provides programming examples for illustration only,
//'' This sample program assumes that you are familiar with the programming
//'' language being demonstrated and the tools used to create and debug
//'' procedures. Agilent Technologies support engineers can help explain the
//'' functionality of Agilent Technologies software components and associated
//'' commands, but they will not modify these samples to provide added
//'' functionality or construct procedures to meet your specific needs.
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

namespace FileManageDMM
{
    class FileManage
    {
        /*Four use cases using the MMEM subsystem. This subsystem contains the SCPI file management commands that can be used to create/copy/paste files.
         * 1) Takes 10 readings. Then stores the data from reading memory and copies it to a file on the internal DMM drive. It then shows how to copy
         * that data over to the host PC. 
         * 2) Takes the current state of the DMM then saves the preference and state into files on the internal DMM drive. It then resets the DMM and recalls
         * the state and preference files to bring back the previous state of the DMM
         * 3) we transfer files from the PC to the DMM internal drive and the USB stick (if there is a USB stick installed)
         * 4) Erase all the files that were created.
         */

        /*'This program does not exhibit the most robust error handling. This means that there could be cases where the program will not run due to file errors. This
        'is done on purpose to keep the program simple so that you can read and understand it.
         * */
        static void Main(string[] args)
        {
            FileManage DmmClass = new FileManage(); //Create an instance of this class so we can call functions from Main

            //For more information on getting started using VISA COM I/O operations see the app note located at:
            //http://cp.literature.agilent.com/litweb/pdf/5989-6338EN.pdf

            Ivi.Visa.Interop.ResourceManager rm = new Ivi.Visa.Interop.ResourceManager(); //Open up a new resource manager
            Ivi.Visa.Interop.FormattedIO488 myDmm = new Ivi.Visa.Interop.FormattedIO488(); //Open a new Formatted IO 488 session 
            try
            {
                //string DutAddr = "GPIB0::22"; //String for GPIB
                //string DutAddr = "TCPIP0::169.254.4.61";  //Example string for LAN
                //string DutAddr = "USB0::0x0957::0x1A07::MY53000101"; //Example string for USB
                //string DutAddr = "USB0::0x0957::0x1C07::US00000069::0::INSTR"; //String for GPIB
                string DutAddr = "TCPIP0::156.140.92.16";  //Example string for LAN

                myDmm.IO = (IMessage)rm.Open(DutAddr, AccessMode.NO_LOCK, 2000, ""); //Open up a handle to the DMM with a 5 second timeout
                myDmm.IO.Timeout = 5000; 

                //First start with a reset state
                myDmm.IO.Clear(); //Send a device clear first to stop any measurements in process
                myDmm.WriteString("*RST", true); //Reset the device
                myDmm.WriteString("*IDN?", true); //Get the IDN string                
                string IDN = myDmm.ReadString();
                Console.WriteLine(IDN); //report the DMM's identity

                /*Example 1: Take 10 resistance readings and report them to the screen. 
                 * Then save the readings from the buffer into an internal file on the DMM. 
                 * Finally transfer the file from the DMM to the PC and store in C:\Temp
                 * */
                Console.WriteLine("Example 1: Get readings from DMM, also store readings from DMM into a file on DMM");
                //Configure for Resistance 1000ohm range, 10mV resolution this is a fast reading
                myDmm.WriteString("CONF:RES 1000, 0.01", true);
                myDmm.WriteString("SAMP:COUN 10", true); //Get 10 Readings per trigger
                myDmm.WriteString("READ?", true); //Trigger the DMM to start the readings, can also use INIT and FETCH commands
                Object[] OhmResult = (object[])myDmm.ReadList(IEEEASCIIType.ASCIIType_Any, ",;"); //Get the readings in an array

                Console.WriteLine("\nReadings direct from DMM: ");

                foreach (double rdg in OhmResult) //Write out each result to the console
                    Console.WriteLine(rdg.ToString());

                //Check for any DMM errors
                DmmClass.CheckDMMError(myDmm);

                //Transfer/Save the readings into Internal disk
                myDmm.WriteString("MMEM:STOR:DATA RDG_STORE, \"INT:\\MyOhmsRdgs\"", true); //Creates and stores readings into MyOhmssRdgs.csv

                //Let's read out the readings from the file into the console
                myDmm.WriteString("MMEM:UPL? \"INT:\\MyOhmsRdgs.csv\"", true);

                
                
                //The readings are now in a file, so we need to read out as bytes, then convert to a string for printout
                byte[] FileRdgs = (byte[])myDmm.ReadIEEEBlock(IEEEBinaryType.BinaryType_UI1,false,true); 
                string StrFileRdgs = System.Text.Encoding.UTF8.GetString(FileRdgs);
                Console.WriteLine("\nReadings from File:");
                Console.WriteLine(StrFileRdgs);

                //Check for any DMM errors
                DmmClass.CheckDMMError(myDmm);

                //Now that we have the file read back, let's save the file on our PC
                string OutFilename = "C:\\Temp\\OhmsReadings.txt"; //Set file location
                System.IO.FileStream f = System.IO.File.Open(OutFilename, System.IO.FileMode.Create); //Create/Overwrite the local copy of the file

                f.Write(FileRdgs, 0, FileRdgs.Length); //Write the file data
                f.Close(); //Close the file
                Console.WriteLine("Ohms readings saved at: " + OutFilename);
                Console.WriteLine("Press any key to continue to Example 2");
                Console.ReadKey();
                /*Example 2
                 * Take the current state of the DMM then saves the preference and state into files on the internal DMM drive. 
                 * Then resets the DMM and recalls the state and preference files to bring back the previous state of the DMM
                 *      State files contain measurement specific instrument states such as Function, range, trigger settings etc.
                 *      Preference files contain DMM system settings such as IO addresses, display brightness, power-on state etc.
                 */
                Console.WriteLine("Example 2: Create State files from DMM, reset DMM, then recall the state from the State files");
                myDmm.WriteString("CONF?", true);
                string currConf = myDmm.ReadString();
                Console.WriteLine("Current Configuration is: " + currConf);
                myDmm.WriteString("MMEM:STOR:STAT \"INT:\\OhmsStat\"", true); //Store the DMM state into an internal file, it automatically stores as .sta file
                myDmm.WriteString("MMEM:STOR:PREF \"INT:\\OhmsPref\"", true); //Store the DMM preferences into an internal file, automatically stores as .prf file. 

                //reset the DMM and prove that the state has changed
                myDmm.WriteString("*RST", true);
                myDmm.WriteString("CONF?", true);
                string RstConf = myDmm.ReadString();
                Console.WriteLine("Reset Configuration is: " + RstConf);

                //Load the state and preference files from out previous state
                Console.WriteLine("Recalling State and preferences into DMM");
                myDmm.WriteString("MMEM:LOAD:STAT \"INT:\\OhmsStat.sta\"", true); //Recall the DMM state from the internal file
                //myDmm.WriteString("MMEM:LOAD:PREF \"INT:\\OhmsPref.prf\""); //Recall the DMM preferences from the internal file, this is commented out because it will reboot the DMM.

                myDmm.WriteString("CONF?", true);
                string LoadConf = myDmm.ReadString();
                Console.WriteLine("Reloaded Configuration is: " + LoadConf);
                Console.WriteLine("Press any key to continue to Example 3");
                Console.ReadKey();

                /*Example 3: Transfer files from PC to DMM internal drive and USB stick. 
                 * Get the OhmsReading.txt file store in C:\Temp from the previous example and copy it to a new directory on the DMM drive
                 * Check if the DMM has a USB drive, if not then do a single prompt to install it. If the second check doesn't find a USB stick skip the USB stick portion
                 * If the USB stick is installed then create a new folder and copy the File to the USB drive.
                 * Finally, delete all of the files that were created from this example.*/

                Console.WriteLine("Example 3: Copy files from PC to DMM");

                //Create a new folder on the DMM
                myDmm.WriteString("MMEM:MDIR \"INT:\\Rdgs\"", true);
//
                //Specifying a filename on the DMM to download our new file to
                myDmm.WriteString("MMEM:DOWN:FNAM \"INT:\\Rdgs\\MyOhmsRdgs.csv\"", true);

                //Check DMM Errors
                DmmClass.CheckDMMError(myDmm);

                //Read out the file from the PC
                System.IO.FileStream OhmFile = System.IO.File.Open(OutFilename, System.IO.FileMode.Open); //Open the local copy of the file
                byte[] OhmByte = new byte[OhmFile.Length];
                OhmFile.Read(OhmByte,0, (int)OhmFile.Length); //Read the file data
                OhmFile.Close(); //Close the file

                myDmm.WriteIEEEBlock("MMEM:DOWN:DATA ", OhmByte, true); //Download the data in IEEE format
                //Check DMM Errors
                DmmClass.CheckDMMError(myDmm);

                //Check if the file was downloaded correctly
                myDmm.WriteString("MMEM:CAT? \"INT:\\Rdgs\"", true);
                string CatDMM = myDmm.ReadString();
                Console.WriteLine("INT:\\Rdgs\\ contains: "+CatDMM);
                

                //Now let's do the same operations for the USB Drive
                
                //First Check if there is a USB drive installed on the DMM
                bool DoUSBtest = DmmClass.CheckForUSB(myDmm);  //Check if we want to do the USB test
                if (DoUSBtest) //Then if we want to do the test copy the files over
                {
                    //Create a new folder on the DMM
                    myDmm.WriteString("MMEM:MDIR \"USB:\\Rdgs\"", true);
                    DmmClass.CheckDMMError(myDmm); //Check for errors

                    //Specifying a filename on the DMM to download our new file to
                    myDmm.WriteString("MMEM:DOWN:FNAM \"USB:\\Rdgs\\MyOhmsRdgs.csv\"", true);
                    DmmClass.CheckDMMError(myDmm); //Check for errors
                    
                    //Finally write to the file 
                    myDmm.WriteIEEEBlock("MMEM:DOWN:DATA ", OhmByte, true); //Download the data in IEEE format                    
                    DmmClass.CheckDMMError(myDmm); //Check for errors

                    //Check if the file was downloaded correctly
                    myDmm.WriteString("MMEM:CAT? \"USB:\\Rdgs\"", true);
                    string CatUSB = myDmm.ReadString();
                    Console.WriteLine("USB:\\Rdgs\\ contains: " + CatUSB);
                

                }

                Console.WriteLine("Press any key to delete all files created by this example");
                Console.ReadKey();

            //Example 4
            //Finally Delete all the files and folders that were created

                //Deletes the Ohms reading file from the base directory
                myDmm.WriteString("MMEM:DEL \"INT:\\MyOhmsRdgs.csv\"", true); 

                //Delete the state and preference files that we created
                myDmm.WriteString("MMEM:DEL \"INT:\\OhmsStat.sta\"", true); //Delete the DMM state file
                myDmm.WriteString("MMEM:DEL \"INT:\\OhmsPref.prf\"", true); //Delete the DMM preferences file
                DmmClass.CheckDMMError(myDmm); //Check for error

                //Delete the INT .csv file that we created
                myDmm.WriteString("MMEM:DEL \"INT:\\Rdgs\\MyOhmsRdgs.csv\"", true);  //Deletes the file from the DMM Rdgs directory
                
                DmmClass.CheckDMMError(myDmm); //Check for error

                //Then delete the directory, directory deletion is only allowed if the directory is empty
                myDmm.WriteString("MMEM:RDIR \"INT:\\Rdgs\"", true);
                DmmClass.CheckDMMError(myDmm); //Check for error

                if (DoUSBtest) //Only delete the files if we did the USB test
                {
                    //Delete the USB .csv file that we created
                    myDmm.WriteString("MMEM:DEL \"USB:\\Rdgs\\MyOhmsRdgs.csv\"", true);
                    DmmClass.CheckDMMError(myDmm); //Check for error

                    //Then delete the directory, directory deletion is only allowed if the directory is empty
                    myDmm.WriteString("MMEM:RDIR \"USB:\\Rdgs\"", true);
                    DmmClass.CheckDMMError(myDmm); //Check for error                               
                }
                
                
            }
            catch (Exception e)
            {
                Console.WriteLine("Error occured: " + e.Message);
            }
            finally
            {
                //Close out your resources
                try { myDmm.IO.Close(); }
                catch { }
                try { System.Runtime.InteropServices.Marshal.ReleaseComObject(myDmm); }
                catch { }
                try
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(rm);
                }
                catch { }
                Console.WriteLine("Press any key to continue...");
                Console.ReadKey();

            }


        }
        public void CheckDMMError(FormattedIO488 myDmm)
        {

            myDmm.WriteString("SYST:ERR?", true);
            string errStr = myDmm.ReadString();

            if (errStr.Contains("No error")) //If no error, then return
                return;
            //If there is an error, read out all of the errors and return them in an exception
            else
            {
                string errStr2 = "";
                do
                {
                    myDmm.WriteString("SYST:ERR?", true);
                    errStr2 = myDmm.ReadString();
                    if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                } while (!errStr2.Contains("No error"));
                throw new Exception("Exception: Encountered system error(s)\n" + errStr);
            }

        }
        public bool CheckForUSB(FormattedIO488 myDmm)
        {
            //First check if the USB Stick is on the DMM
            myDmm.WriteString("MMEM:CAT? \"USB:\\\"", true); //Change default directory to USB if we can                                                                     
            string junk1 = myDmm.ReadString();

            myDmm.WriteString("MMEM:CDIR \"USB:\\\"", true); //Change default directory to USB if we can                                                                     

                myDmm.WriteString("SYST:ERR?", true);
                string errStr = myDmm.ReadString();
                if (errStr.Contains("+0")) //if no error then USB is available and continue
                     return true;
                else if (errStr.Contains("-256")) //-256 is the error code for file or folder not found
                {
                    //Clear out any additional errors
                    string errStr2 = "";
                    do
                    {
                        myDmm.WriteString("SYST:ERR?", true);
                        errStr2 = myDmm.ReadString();
                        if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                    } while (!errStr2.Contains("No error"));
                    //Ask the user if they want to continue with USB test
                    Console.WriteLine("No USB thumb drive connected. Please install one. If you choose not to install one, this program will exit");
                    Console.WriteLine("Press any key to continue or Press Escape (Esc) key to skip USB copy of file");
                    
                    ConsoleKeyInfo cki = Console.ReadKey();
                    if (cki.Key == ConsoleKey.Escape)
                        return false;                
                }

                myDmm.WriteString("MMEM:CDIR \"USB:\\\"", true);                
                //Check for an error again, if there is an error -256 then we return a false for USB
                myDmm.WriteString("SYST:ERR?", true);  
                errStr = myDmm.ReadString();
                if (errStr.Contains("+0")) //if no error then USB is available and continue
                    return true;
                else if (errStr.Contains("-256")) //-256 is the error code for file or folder not found
                {
                    //Clear out any additional errors
                    string errStr2 = "";
                    do
                    {
                        myDmm.WriteString("SYST:ERR?", true);
                        errStr2 = myDmm.ReadString();
                        if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                    } while (!errStr2.Contains("No error"));
                    return false;
                }
                else
                {
                    CheckDMMError(myDmm);
                    return false;
                }
                                 
        }
    }
}
